/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 *	This example shows how to use the TDMA signaling mode in remote control 
 *
 *	Program flow
 *
 *	1 - Init handles with CMU
 *	2 - Setup the RF input and attenuation => Adapt the attenuation value to your cables losses
 *	3 - Turn off the CMU signaling mode before configuration of the BST
 *	4 - Setup the BST in TDMA US Cellular band
 *	5 - Setup the BST in TDMA PCS band
 *	6 - Turn on signaling mode TDMA US cellular
 *	7 - Etablish a call in DTCH 991
 *  8 - Test TX & RX
 *  9 - Handoff to DTCH 312
 * 10 - Test TX & RX
 * 11 - Handoff to DTCH 799
 * 12 - Test TX & RX
 * 13 - Handoff to PCS BAND
 * 14 - Test TX & RX
 * 15 - Handoff to DTCH 1999
 * 16 - Test TX & RX
 * 17 - Release the call
 *
 * RSA / JM GAYET
*/

/*
REVISION HISTORY

1.01, 12.02.2001, J. SIMA
    Added revision information to this example file.

1.02, 16.02.2001, J. SIMA
    Added CHECKNAN macro to convert CMU NAN value to IEEE NAN value.
    This fixes nonsense number printout for TXD at PCL powers.

1.03, 12.2002, J. KOMINEK
	Changed name of structures of ERROR_LIMITS to RSCMUK2T_ERROR_LIMITS and
	OVERVIEW to RSCMUK2T_OVERVIEW to accept changes in 3.07.2 RSCMUK2T CVI Driver

*/

#include <ansi_c.h>
#include "RSCMU200.h"
#include "RSCMUK2T.h"
#include <utility.h> 

#define IEEE_NAN (*((double*)("\0\0\0\0\0\0\xF8\x7F")))
#define CHECKNAN(power) (power>RSCMU_NAN/2.0) ? IEEE_NAN : power

// declaration
struct t_DCCH {

	int			Channel;
	double		RFLev;
	int			TS;
	
} DCCH;

struct t_DTCH {

	int			Channel;
	double		RFLev;
	int			TS;

	
} DTCH;


struct t_NETW {

	int			SID;
	int			DCCHMAC;
	int			DTCHMAC;
	int			VOCODER;
	int			SPEECHMODE;
	int			DVCC;
	
} NETW;

struct	t_HANDOFF {

	int			DTCH;
	double		DTCH_RFLev;
	int			DTCH_MAC;
	int			DTCH_TS;
	
} HANDOFF;


struct t_MS {
	
	int			SCM;
	char		ESN[20];
	char		MIN[20];
	
} MS;

struct t_pow {

	ViInt32 	BMAvg;
	ViReal64 	BOT;
	ViInt32 	BMMax;
	ViInt32 	BMMin;
	ViReal64 	PowerPKLK;
	ViReal64 	PowerRMSLK;
	ViInt32 	BMCur;
	ViReal64 	PowerPK;
	ViReal64 	PowerRMS;
	
} TXD_POW;


struct t_RX {

	int		RSSI;
	int		BER;
	int		BSTLev;
	
} RXD;

	ViSession		idBASE;
	ViSession		idTDMA800;	
	ViSession		idTDMA1900;
	ViStatus 		cmu_errcode = 0;
	ViStatus		cmu_status	=0;

 	ViChar 			buffin[RD_BUFFER_SIZE];
	ViChar			cmu_errmsg[1024];


	ViInt32 		state;
	ViInt32    		_error;

	int				TestLoop 	= 0;
	int				c;
	int				i			= 0;
	int				flag1 		= 0;
	int				flag2 		= 0;
	int				pow_status 	= 0;
	int				acp_status	= 0;
	int				rssi_status = 0;
	int				cycle;
	int				counter;
	int				rssi_tolp;
	int				rssi_tolm;

	RSCMUK2T_ERROR_MOD		TXD_MOD;
	RSCMUK2T_OVERVIEW		TXD_MOD_OVER;
	SCALAR_ACP		TXD_ACP;
	MAHO			RXD_MAHO[24];
	
  	clock_t    		StartTime;
  	clock_t    		CurrentTime;
	clock_t			TimeOut;

	double			power;
	int				maho_channel_list[24] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
					
// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif


// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
	ViChar l_vCmuErrMsg[256];
	ViChar l_vErrBuf[512];
	ViInt32 _error;
    
	if (CmuStatus != VI_SUCCESS)
	{

		//RSCMU_Base_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
		RSCMU_Base_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);

		//Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
		sprintf (l_vErrBuf, "Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);        
		printf ("ERROR MESSAGE (CMU)\n %s\n", l_vErrBuf);
	
	return(-1);
	}
    return(0);
    
Error:
    return(-1);
}


//
//	Function set_RF_path
//
int	set_RF_path(ViSession handle, ViInt32 rf_inp, ViReal64 rf_attinp, ViInt32 rf_outp,  ViReal64 rf_attoutp) {

	int		Error;

	CheckCmuError(RSCMU_TDMA_InpState (handle, rf_inp, cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_OutpState (handle, rf_outp, cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_SenseCorrLossInp (handle, rf_inp, rf_attinp, cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_SenseCorrLossOutp (handle, rf_outp, rf_attoutp,cmu_errmsg));


	return(0);
	
Error:
  return (-1);
	

}

//
// Function :  Setup_BTS
//				Configure Base Station RF
//				Configure Network
int Setup_BTS(ViSession handle, struct t_DCCH DCCH, struct t_DTCH DTCH, struct t_NETW NETW, struct t_HANDOFF HANDOFF)
{ 

	int		Error;

	// Setup BASE station 
	//	DCCH & DTCH 
	//	SID
	CheckCmuError(RSCMU_TDMA_ConfBSGlobal (handle
											,DCCH.Channel
											,DCCH.TS
											,NETW.DCCHMAC
											,DCCH.RFLev
											,DTCH.Channel
											,RSCMU_TCH_C14
											,NETW.DTCHMAC
											,DTCH.RFLev
											,NETW.SID
											,NETW.VOCODER
											,NETW.DVCC
											,cmu_errmsg));
											
	// SPEECH MODE
	CheckCmuError(RSCMU_TDMA_ConfBSSignalSpeech (handle, RSCMU_ECHO, cmu_errmsg));
	
	
	// Setup HANDOFF
	CheckCmuError(RSCMU_TDMA_ConfHandoffTDMA (handle
											,HANDOFF.DTCH
											,HANDOFF.DTCH_RFLev
											,HANDOFF.DTCH_MAC
											,HANDOFF.DTCH_TS
											,cmu_errmsg));
	
	
	return(0);
	
Error:
  return (-1);
	
}

int setup_meas( ViSession handle, int TX_pow_NB, int TX_mod_NB, int TX_ACP_NB, int maho_list[]) {

	int 	i;
	
	// Configure TX peak power object
	printf("\tConfigure TX peak power object\n");	
	CheckCmuError(RSCMU_TDMA_SPowerContRep (handle, RSCMU_SINGLE_SHOT, RSCMU_NONE,RSCMU_NONE, cmu_errmsg));
	
	// Configure TX power PVT object
	printf("\tConfigure TX power object - N Burst(s) : %d \n", TX_pow_NB);		
	CheckCmuError(RSCMU_TDMA_ConfPowCont (handle, RSCMU_NORMAL, RSCMU_SCALAR, TX_pow_NB,RSCMU_OVERSAMPLE1, cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_ConfPowContRep (handle, RSCMU_NORMAL, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, cmu_errmsg));
	
	
	// Configure TX Modulation object 
	printf("\tConfigure TX modulation object\n\t\tOverview - N Burst(s) : %d \n", TX_mod_NB);	
	CheckCmuError(RSCMU_TDMA_ConfModCont (handle, RSCMU_OVERVIEW, RSCMU_SCALAR, TX_mod_NB,cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_ConfModContRep (handle, RSCMU_OVERVIEW, RSCMU_SINGLE_SHOT,RSCMU_NONE, RSCMU_NONE, cmu_errmsg));
	
	
	printf("\t\tEVM - N Burst(s) : %d \n", TX_mod_NB);		
	CheckCmuError(RSCMU_TDMA_ConfModCont (handle, RSCMU_EVM, RSCMU_SCALAR, TX_mod_NB, cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_ConfModContRep (handle, RSCMU_EVM, RSCMU_SINGLE_SHOT,RSCMU_NONE, RSCMU_NONE, cmu_errmsg));

	printf("\t\tMER - N Burst(s) : %d \n", TX_mod_NB);		
	CheckCmuError(RSCMU_TDMA_ConfModCont (handle, RSCMU_MERROR, RSCMU_SCALAR, TX_mod_NB,cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_ConfModContRep (handle, RSCMU_MERROR, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, cmu_errmsg));

	printf("\t\tPER - N Burst(s) : %d \n", TX_mod_NB);		
	CheckCmuError(RSCMU_TDMA_ConfModCont (handle, RSCMU_PERROR, RSCMU_SCALAR, TX_mod_NB,cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_ConfModContRep (handle, RSCMU_PERROR, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, cmu_errmsg));
	
	// Configure TX ACP object 	
	printf("\tConfigure TX ACP object - N Burst(s) : %d \n", TX_ACP_NB);
	CheckCmuError(RSCMU_TDMA_ConfSpecContRep (handle, RSCMU_SINGLE_SHOT, RSCMU_NONE,RSCMU_NONE, cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_ConfSpecStatBurst (handle, TX_ACP_NB, cmu_errmsg));
	
	
	// Configure RX MAHO List
	i = 0;
	while (maho_list[i] != -1){
		
		CheckCmuError(RSCMU_TDMA_ConfNetNeighborChan (handle, i+1, maho_list[i], cmu_errmsg));
		i++;
	};
	
	return(0);
	
Error:
  return (-1);

}

//
// Function main
//
int main( void ) {

	Cls();
	
	// 	Open handles
	//	Open handle with CMU base
	printf("\nOpen handles with the CMU\n");	
	CheckCmuError(RSCMU_Init_Base (&idBASE, RSCMU_GPIB, 0, 20,RSCMU_DO_QUERY, RSCMU_RESET_ON, cmu_errmsg));
	

	// Open handle with the TDMA US Cellular
	printf("\tOpen handle with the BASE\n");	
	CheckCmuError(RSCMU_Init_TDMA (idBASE, &idTDMA800, RSCMU_IS136800MS_SIG, cmu_errmsg));


	// Open handle with the TDMA US PCS
	printf("\tOpen handle with the TDMA800 signaling mode\n");
	CheckCmuError(RSCMU_Init_TDMA (idBASE, &idTDMA1900, RSCMU_IS1361900MS_SIG, cmu_errmsg));
	
	
	// Check instrument state and turn off signaling
	// Test signaling in US band
	printf("\tOpen handle with the TDMA1900 signaling mode\n");	
	CheckCmuError(RSCMU_TDMA_ProcSigState (idTDMA800, &state, cmu_errmsg));
	
	printf("\nTurn off signaling mode\n");
	if (state != 0)   CheckCmuError(RSCMU_TDMA_ProcSigAction (idTDMA800, RSCMU_SOFF, cmu_errmsg));
		
	// Test signaling in PCS band
	CheckCmuError(RSCMU_TDMA_ProcSigState (idTDMA1900, &state, cmu_errmsg));
	
	if (state != 0)   CheckCmuError(RSCMU_TDMA_ProcSigAction (idTDMA1900, RSCMU_SOFF, cmu_errmsg));
	
	
	// Setup RF path US band
	printf("\nConfigure RF path and RF attenuation\n");
	if (set_RF_path(idTDMA800, RSCMU_INPUT_RF2, 5.0, RSCMU_OUTPUT_RF2, 5.0) != 0) return -1;

	// Setup RF path PCS band	
	if (set_RF_path(idTDMA1900, RSCMU_INPUT_RF2, 5.0, RSCMU_OUTPUT_RF2, 5.0) != 0) return -1;	
	
	// Setup Base station in US cellular band
	//		Setup BCCH
	printf("\nConfigure BST parameters in TDMA US cell band\n");	
	DCCH.Channel 	= 997;								// Channel
	DCCH.RFLev		= -50.0;							// BCCH RF lev
	DCCH.TS			= RSCMU_CCH_C14;					// Full rate TS 1&4
	//		Setup TCH	
	DTCH.Channel	= 991;								// Channel
	DTCH.RFLev		= -75.0;							// DTCH RF lev
	DTCH.TS			= RSCMU_TCH_C14;					// DTCH Time SLot
	
	// Network parameters
	NETW.SID		= 61;								// SID
	NETW.DCCHMAC	= 6;								// DCCH MAC
	NETW.DTCHMAC	= 2;								// DTCH	MAC
	NETW.VOCODER	= RSCMU_ACELP;						// Vocoder ACELP
	NETW.SPEECHMODE = RSCMU_ECHO;						// SPEECH LOOP BACK MODE WITH ECHO
	NETW.DVCC		= 255;
	
	// HANDOFF TO US Cell band
	HANDOFF.DTCH	= 200;
	HANDOFF.DTCH_RFLev	= -75.0;
	HANDOFF.DTCH_TS	= RSCMU_TCH_C14;
	HANDOFF.DTCH_MAC	= 2;
	
	if (Setup_BTS(idTDMA800, DCCH, DTCH, NETW, HANDOFF) != 0) return -1;

	// Setup Base station in PCS cellular band
	//		Setup BCCH
	printf("\nConfigure BST parameters in TDMA PCS cell band\n");		
	DCCH.Channel 	= 200;								// Channel
	DCCH.RFLev		= -50.0;							// BCCH RF lev
	DCCH.TS			= RSCMU_CCH_C14;					// Full rate TS 1&4
	//		Setup TCH	
	DTCH.Channel	= 2;								// Channel
//M.K.	DTCH.RFLev		= -75.0;							// DTCH RF lev
	DTCH.RFLev		= -85.0;							// DTCH RF lev
	DTCH.TS			= RSCMU_TCH_C14;					// DTCH Time SLot
	
	NETW.SID		= 61;								// SID
	NETW.DCCHMAC	= 6;								// DCCH MAC
	NETW.DTCHMAC	= 2;								// DTCH	MAC
	NETW.VOCODER	= RSCMU_ACELP;						// Vocoder ACELP
	NETW.SPEECHMODE = RSCMU_ECHO;						// SPEECH LOOP BACK MODE WITH ECHO
	NETW.DVCC		= 255;
	
	// HANDOFF TO US PCS band	
	HANDOFF.DTCH	= 2;
	HANDOFF.DTCH_RFLev	= -75.0;
	HANDOFF.DTCH_TS	= RSCMU_TCH_C14;
	HANDOFF.DTCH_MAC	= 2;
	
	if (Setup_BTS(idTDMA1900, DCCH, DTCH, NETW, HANDOFF) != 0) return -1;
	
	maho_channel_list[0] = 991;
	maho_channel_list[1] = 1023;
	maho_channel_list[2] = 40;
	maho_channel_list[3] = 100;
	maho_channel_list[4] = 200;
	maho_channel_list[5] = 312;
	maho_channel_list[6] = 331;
	maho_channel_list[7] = 333;
	maho_channel_list[8] = 334;
	maho_channel_list[9] = 400;
	maho_channel_list[10] = 550;
	maho_channel_list[11] = 600;
	maho_channel_list[12] = 716;
	maho_channel_list[13] = 799;
	
	// Setup Measurement for TDMA US Cell
	if ( setup_meas( idTDMA800, 10, 10 , 25, maho_channel_list) != 0) return -1;
	
	for(i = 0; i<24; i++) maho_channel_list[i] = -1;
	if ( setup_meas( idTDMA1900, 10, 10 , 25, maho_channel_list) != 0) return -1;
	
	// 
	// Turn on signalling 
	printf("\nTurn on signaling mode in TDMA 800 cell\n");		
	CheckCmuError(RSCMU_TDMA_ProcSigAction (idTDMA800, RSCMU_SON, cmu_errmsg));

	TestLoop = 1;
	do
	{

		// Display Message and Query item
		printf("\n\nFor a new test \tHit 1<Enter>\nFor exit \t\thit 0<Enter>\n\t\tEnter item : ");
		scanf("%d", &c);
      
		if (c != 1) {
		 TestLoop = 0;
		 break;
		};
   
		// Query user 
		while (c = getchar() != EOF) {
		   printf("\n(1)Switch ON MS and Wait for service available\n(2)press <CTRL>Z to continue\n");

		};
		
		
		// Registration
		printf("\n\tRegistration in progres ...\n");
		cmu_status = RSCMU_TDMA_Registration (idTDMA800, 15, cmu_errmsg);
		if (cmu_status != 0) {
			printf("Registration failed\nError code :%x\nError Message : %s\n", cmu_status, cmu_errmsg);
			continue;
		}
		printf("\tRegistration successfull ...\n");
		
		// Page MS
		printf("\n\tPaging to MS in progres ...\n");		
		cmu_status = RSCMU_TDMA_PagingCallToMS (idTDMA800, 15, cmu_errmsg);
		if (cmu_status != 0) {
			printf("Paging failed\nError code :%x\nError Message : %s\n", cmu_status, cmu_errmsg);
			continue;
		}
		
		// TEST MS in a call
		cmu_status = RSCMU_TDMA_ProcSigState (idTDMA800, &state, cmu_errmsg);
		if ( state != 4) {
			printf("Mobile Not in a call\n");
			continue;
		}
		printf("\tMS in a call\n");
		
		// Query MS info 
		printf("\nQuery MS info\n");
		CheckCmuError(RSCMU_TDMA_SenseMobileInfoESN (idTDMA800, MS.ESN, cmu_errmsg));
		CheckCmuError(RSCMU_TDMA_SenseMobileInfoPowClass (idTDMA800, &MS.SCM, cmu_errmsg));
		printf("\tSCM\t\t: %d\n", MS.SCM);
		printf("\tESN\t\t: %s\n", MS.ESN);

		
		// TEST IN DTCH 991
		printf("\nTEST IN US CELL BAND\n");		
		printf("\nTEST in CHANNEL 991\n");
		
		// TXD Modulation
		printf("\tTXD modulation over 10 first symbols  & RMS power\n");
		
		CheckCmuError(RSCMU_TDMA_ReadMod (idTDMA800, RSCMU_OVERVIEW, RSCMU_F10SYMBOLS,&TXD_MOD, &TXD_MOD_OVER, cmu_errmsg));
		printf("\tTXD Power RMS\t\t: %6.2f dBm\n", TXD_MOD_OVER.PowerRMS);
		printf("\tTXD Freq Error\t\t: %6.0f HZ\n", TXD_MOD_OVER.FreqErr.Average);
		printf("\tTXD EVM10 RMS\t\t: %6.2f %%\n", TXD_MOD_OVER.EVMRMS.Average);
		printf("\tTXD EVM10 Peak\t\t: %6.2f %%\n", TXD_MOD_OVER.EVMPeak.MaxMin);
		printf("\tTXD MERR RMS\t\t: %6.2f dB\n", TXD_MOD_OVER.MErrRMS.Average);
		printf("\tTXD MERR Peak\t\t: %6.2f dB\n", TXD_MOD_OVER.MErrPeak.MaxMin);
		printf("\tTXD PERR RMS\t\t: %6.2f Deg\n", TXD_MOD_OVER.PErrRMS.Average);
		printf("\tTXD PERR Peak\t\t: %6.2f Deg\n", TXD_MOD_OVER.PErrPeak.MaxMin);
		printf("\tTXD IQ imbalance\t: %6.2f dB\n", TXD_MOD_OVER.IQImbalance.Average);
		printf("\tTXD Origin Offset\t: %6.2f dB\n", TXD_MOD_OVER.OriginOffset.Average);		
		printf("\tTXD Amplitude droop\t: %6.2f dB\n", TXD_MOD_OVER.AmplitudeDrop);
		printf("\tTXD Timing error\t: %6.2f Symb\n", TXD_MOD_OVER.TimingErr);
		
		CheckCmuError(RSCMU_TDMA_AbortMod (idTDMA800, RSCMU_OVERVIEW, cmu_errmsg));
		
		// TXD Power VT and RSSI in parallel
		printf("\n\tTXD power PVT & RXD RSSI in parallel\n");
		
		// 	Start TX PVT
		CheckCmuError(RSCMU_TDMA_InitPow (idTDMA800, RSCMU_NORMAL, cmu_errmsg));
		
		flag1 		= 0;
		flag2 		= 0;
		pow_status 	= 0;
		rssi_status = 0;
		
		rssi_tolp = ((113 + (int) DTCH.RFLev)/2) + 3;
		rssi_tolm = ((113 + (int) DTCH.RFLev)/2) - 3;
		
//M.K.	TimeOut		= 5.0;
		TimeOut		= 10.0;
		StartTime	= clock() / CLOCKS_PER_SEC;
		do {
		
			if (flag1 == 0) {
			
				//	Query TX Power if status ok
				if (pow_status == 5) {
					CheckCmuError(RSCMU_TDMA_FetchScalPow (idTDMA800
											,RSCMU_NORMAL
											,&TXD_POW.BOT
											,&TXD_POW.PowerPK
											,&TXD_POW.PowerRMS
											,&TXD_POW.PowerPKLK
											,&TXD_POW.PowerRMSLK
											,&TXD_POW.BMCur
											,&TXD_POW.BMMin
											,&TXD_POW.BMMax
											,&TXD_POW.BMAvg
											,cmu_errmsg));
														
					flag1 = 1;
					pow_status = 0;
				}
				else
					// Test the power object status
					CheckCmuError(RSCMU_TDMA_FetchPowStatus (idTDMA800, RSCMU_NORMAL, &pow_status,&cycle, &counter, cmu_errmsg));
			
				
			}
			
			if (flag2 == 0) {
				
				//	Query RX RSSI if status ok
				if (rssi_status == 5)  {
					CheckCmuError(RSCMU_TDMA_SenseChanQualityRSSI (idTDMA800, &RXD.RSSI, &RXD.BSTLev,cmu_errmsg));
					CheckCmuError(RSCMU_TDMA_SenseChanQualityBER (idTDMA800, &RXD.BER, cmu_errmsg));
					
					flag2 = 1;
					rssi_status = 0;
				}
				else {
					// Test the RSSI
					CheckCmuError(RSCMU_TDMA_SenseChanQualityRSSI (idTDMA800, &RXD.RSSI, &RXD.BSTLev,
													 cmu_errmsg));
					// Test RSSI limit
					if ((RXD.RSSI >= rssi_tolm) && (RXD.RSSI <= rssi_tolp) )
						rssi_status  	= 5;
					else
						rssi_status		= 0;
					
				}
			
			}
			
			CurrentTime = clock() / CLOCKS_PER_SEC;
			if (CurrentTime > StartTime + TimeOut) CheckCmuError(RSCMU_ERROR_TIMEOUT);

		} while (!((flag1 == 1) && (flag2 == 1) )); 

		printf("\tTXD Power RMS\t\t: %6.2f dBm\n", TXD_POW.PowerRMS);
		printf("\tTXD Power Peak\t\t: %6.2f dBm\n", TXD_POW.PowerPK);		
		printf("\tTXD Power Leak RMS\t: %6.2f dBm\n", TXD_POW.PowerRMSLK);
		printf("\tTXD Power Leak Peak\t: %6.2f dBm\n", TXD_POW.PowerPKLK);
		printf("\tTXD Burst OutOfTol\t: %6.2f %%\n", TXD_POW.BOT);
		printf("\tTXD Burst Cur Match\t: %6d %%\n", TXD_POW.BMCur);
		printf("\tTXD Burst Avg Match\t: %6d %%\n", TXD_POW.BMAvg);
		printf("\tTXD Burst Max Match\t: %6d %%\n", TXD_POW.BMMax);
		printf("\tTXD Burst Min Match\t: %6d %%\n", TXD_POW.BMMin);										
		printf("\n\tRXD RSSI\t\t\t: %6d\n", RXD.RSSI);
		printf("\tRXD BST RF Lev\t\t: %6d\n", RXD.BSTLev);
		printf("\tRXD BER\t\t\t\t: %6d\n", RXD.BER);
		// Abort Power PVT
		CheckCmuError(RSCMU_TDMA_AbortPow (idTDMA800, RSCMU_NORMAL, cmu_errmsg));
		
		

		// HANDOFF to DTCH 312
		printf("\nHANDOFF to DTCH 312\n");
		CheckCmuError(RSCMU_TDMA_ProcSigTrafficChan (idTDMA800, 312, cmu_errmsg));
		cmu_status = RSCMU_TDMA_ProcSigState (idTDMA800, &state, cmu_errmsg);
		if ( state != 4) {
			printf("Mobile Not in a call\n");
			continue;
		}

		// TEST IN DTCH 312
		printf("\nTEST in CHANNEL 312\n");

		
		// TXD ACP and RSSI in parallel
		printf("\n\tTXD ACP & RXD RSSI in parallel\n\n");
		
		// 	Start ACP
		CheckCmuError(RSCMU_TDMA_InitSpec (idTDMA800, cmu_errmsg));
		
		flag1 		= 0;
		flag2 		= 0;
		acp_status 	= 0;
		rssi_status = 0;
		
		rssi_tolp = ((113 + (int) DTCH.RFLev)/2) + 3;
		rssi_tolm = ((113 + (int) DTCH.RFLev)/2) - 3;
		
		TimeOut		= 5.0;
		StartTime	= clock() / CLOCKS_PER_SEC;
		do {
		
			if (flag1 == 0) {
			
				//	Query TX Power if status ok
				if (acp_status == 5) {
					CheckCmuError(RSCMU_TDMA_FetchScalSpec (idTDMA800, &TXD_ACP, cmu_errmsg));
														
					flag1 = 1;
					acp_status = 0;
				}
				else
					// Test the power object status
					CheckCmuError(RSCMU_TDMA_FetchSpecStatus (idTDMA800, &acp_status, &counter, &cycle,
															  cmu_errmsg));
			
				
			}
			
			if (flag2 == 0) {
				
				//	Query RX RSSI if status ok
				if (rssi_status == 5)  {
					CheckCmuError(RSCMU_TDMA_SenseChanQualityRSSI (idTDMA800, &RXD.RSSI, &RXD.BSTLev,cmu_errmsg));
					CheckCmuError(RSCMU_TDMA_SenseChanQualityBER (idTDMA800, &RXD.BER, cmu_errmsg));
					
					flag2 = 1;
					rssi_status = 0;
				}
				else {
					// Test the RSSI
					CheckCmuError(RSCMU_TDMA_SenseChanQualityRSSI (idTDMA800, &RXD.RSSI, &RXD.BSTLev,
													 cmu_errmsg));
					// Test RSSI limit
					if ((RXD.RSSI >= rssi_tolm) && (RXD.RSSI <= rssi_tolp) )
						rssi_status  	= 5;
					else
						rssi_status		= 0;
					
				}
			
			}
			
			CurrentTime = clock() / CLOCKS_PER_SEC;
			if (CurrentTime > StartTime + TimeOut) CheckCmuError(RSCMU_ERROR_TIMEOUT);
			
			
		} while (!((flag1 == 1) && (flag2 == 1) )); 

		printf("\tTXD Power RMS\t\t: %6.2f dBm\n", TXD_ACP.RMS[0]);
		printf("\tTXD ACP Average RMS & Peak\n\t");
		for(i = -3; i < 0; i++) printf("  %2d   ",i);
		for(i = 1; i < 4; i++) printf("  %2d   ",i);
		printf("\n\t");
		for(i = 0; i < 6; i++) printf("%6.2f ",TXD_ACP.AverageRMS[i]);
		printf("\n\t");		
		for(i = 0; i < 6; i++) printf("%6.2f ",TXD_ACP.PeakPeak[i]);
		printf("\n");		
		printf("\n\tRXD RSSI\t\t\t: %6d\n", RXD.RSSI);
		printf("\tRXD BST RF Lev\t\t: %6d\n", RXD.BSTLev);
		printf("\tRXD BER\t\t\t\t: %6d\n", RXD.BER);
		
		// Abort ACP
		CheckCmuError(RSCMU_TDMA_AbortSpec (idTDMA800, cmu_errmsg));
		
		
		
		// HANDOFF to DTCH 799
		printf("\nHANDOFF to DTCH 799\n");
		CheckCmuError(RSCMU_TDMA_ProcSigTrafficChan (idTDMA800, 799, cmu_errmsg));
		cmu_status = RSCMU_TDMA_ProcSigState (idTDMA800, &state, cmu_errmsg);
		if ( state != 4) {
			printf("Mobile Not in a call\n");
			continue;
		}
		
		printf("\nTEST in CHANNEL 799\n");

		// TXD Peak power
		printf("\n\tTXD Peak power at all PCL\n\n");

		CheckCmuError(RSCMU_Delay (3.0)); // M.K.
		
		for( i = 2; i < 11; i++) {
		
			CheckCmuError(RSCMU_TDMA_ProcSigDtcMac (idTDMA800, i, cmu_errmsg));
			printf("\tTXD at PCL%d ",i);
			
			CheckCmuError(RSCMU_TDMA_ReadSPowerResult (idTDMA800, &power, cmu_errmsg));
			printf("Power\t: %6.2f dBm\n", CHECKNAN(power));
		}
		CheckCmuError(RSCMU_TDMA_AbortSPower (idTDMA800, cmu_errmsg));
		

		// MAHO
		CheckCmuError(RSCMU_TDMA_SenseChanQualityMAHONCells (idTDMA800, RXD_MAHO,cmu_errmsg));
		printf("\n");		
		printf("\n\tRXD RSSI\t\t\t: %6d\n", RXD_MAHO[13].RSSI);
		printf("\tRXD BST RF Lev\t\t: %6.2f\n", RXD_MAHO[13].Level);

		
		// Check IF MS is a dual band US cell and PCS
		RSCMU_Query (idTDMA800, "STATUS:HANDOFF:TARGET:LIST?", buffin, RSCMU_OFF, cmu_errmsg);
		
		if (strstr (buffin, "IS1361900MS") != NULL) {
		
			// HANDOFF to PCS band
			printf("\nHANDOFF to PCS BAND\n");		
			CheckCmuError(RSCMU_TDMA_Handoff (idTDMA800, idTDMA1900,RSCMU_HANDOFF_IS1361900MSFallback, 10, cmu_errmsg));
			cmu_status = RSCMU_TDMA_ProcSigState (idTDMA1900, &state, cmu_errmsg);
			if ( state != 4) {
				printf("Mobile Not in a call\n");
				continue;
			}
			printf("\nTEST IN PCS BAND\n");
			
			printf("\nTEST in CHANNEL 2\n");		
		
			// TXD Modulation
			printf("\tTXD modulation over 10 first symbols  & RMS power\n");
		
			CheckCmuError(RSCMU_TDMA_ReadMod (idTDMA1900, RSCMU_OVERVIEW, RSCMU_F10SYMBOLS,&TXD_MOD, &TXD_MOD_OVER, cmu_errmsg));
			printf("\tTXD Power RMS\t\t: %6.2f dBm\n", TXD_MOD_OVER.PowerRMS);
			printf("\tTXD Freq Error\t\t: %6.0f HZ\n", TXD_MOD_OVER.FreqErr.Average);
			printf("\tTXD EVM10 RMS\t\t: %6.2f %%\n", TXD_MOD_OVER.EVMRMS.Average);
			printf("\tTXD EVM10 Peak\t\t: %6.2f %%\n", TXD_MOD_OVER.EVMPeak.MaxMin);
			printf("\tTXD MERR RMS\t\t: %6.2f dB\n", TXD_MOD_OVER.MErrRMS.Average);
			printf("\tTXD MERR Peak\t\t: %6.2f dB\n", TXD_MOD_OVER.MErrPeak.MaxMin);
			printf("\tTXD PERR RMS\t\t: %6.2f Deg\n", TXD_MOD_OVER.PErrRMS.Average);
			printf("\tTXD PERR Peak\t\t: %6.2f Deg\n", TXD_MOD_OVER.PErrPeak.MaxMin);
			printf("\tTXD IQ imbalance\t: %6.2f dB\n", TXD_MOD_OVER.IQImbalance.Average);
			printf("\tTXD Origin Offset\t: %6.2f dB\n", TXD_MOD_OVER.OriginOffset.Average);		
			printf("\tTXD Amplitude droop\t: %6.2f dB\n", TXD_MOD_OVER.AmplitudeDrop);
			printf("\tTXD Timing error\t: %6.2f Symb\n", TXD_MOD_OVER.TimingErr);

			// Abort modulation
			CheckCmuError(RSCMU_TDMA_AbortMod (idTDMA1900, RSCMU_OVERVIEW, cmu_errmsg));
			
			
			// HANDOFF to DTCH 1999
			printf("\nHANDOFF to DTCH 1999\n");
			CheckCmuError(RSCMU_TDMA_ProcSigTrafficChan (idTDMA1900, 1999, cmu_errmsg));
			cmu_status = RSCMU_TDMA_ProcSigState (idTDMA1900, &state, cmu_errmsg);
			if ( state != 4) {
				printf("Mobile Not in a call\n");
				continue;
			}
		
			printf("\nTEST in CHANNEL 1999\n");

			// TXD Peak power
			printf("\n\tTXD Peak power at all PCL\n\n");

			CheckCmuError(RSCMU_Delay (3.0)); // M.K.
			
			for( i = 2; i < 11; i++) {
		
				CheckCmuError(RSCMU_TDMA_ProcSigDtcMac (idTDMA1900, i, cmu_errmsg));
				printf("\tTXD at PCL%d ",i);
			
				CheckCmuError(RSCMU_TDMA_ReadSPowerResult (idTDMA1900, &power, cmu_errmsg));
				printf("Power\t: %6.2f dBm\n", CHECKNAN(power));
			}
			CheckCmuError(RSCMU_TDMA_AbortSPower (idTDMA1900, cmu_errmsg));			
			
			// Release a call
			printf("\nRelease the call from BST...\n");			
			CheckCmuError(RSCMU_TDMA_ReleaseCallFromBST (idTDMA1900, cmu_errmsg));
		
			CheckCmuError(RSCMU_Delay (2.0));
		
			// Test signaling in PCS bandand turn off is on
			CheckCmuError(RSCMU_TDMA_ProcSigState (idTDMA1900, &state, cmu_errmsg));
			if (state != 0)   CheckCmuError(RSCMU_TDMA_ProcSigAction (idTDMA1900, RSCMU_SOFF, cmu_errmsg));

			// Test signaling in US band
			CheckCmuError(RSCMU_TDMA_ProcSigState (idTDMA800, &state, cmu_errmsg));
			if (state == 0)   CheckCmuError(RSCMU_TDMA_ProcSigAction (idTDMA800, RSCMU_SON, cmu_errmsg));
			
		}	
		
		else {
		
			// Release a call
			printf("\nNOT a DUAL BAND MS - Release the call from BST...\n");			
			CheckCmuError(RSCMU_TDMA_ReleaseCallFromBST (idTDMA800, cmu_errmsg));
		
			CheckCmuError(RSCMU_Delay (2.0));
		
		}
		
		
	} while (TestLoop == 1);


	// Check instrument state and turn off signaling
	// Test signaling in US band
	CheckCmuError(RSCMU_TDMA_ProcSigState (idTDMA800, &state, cmu_errmsg));
	if (state != 0)   CheckCmuError(RSCMU_TDMA_ProcSigAction (idTDMA800, RSCMU_SOFF, cmu_errmsg));
		
	// Test signaling in PCS band
	CheckCmuError(RSCMU_TDMA_ProcSigState (idTDMA1900, &state, cmu_errmsg));
	if (state != 0)   CheckCmuError(RSCMU_TDMA_ProcSigAction (idTDMA1900, RSCMU_SOFF, cmu_errmsg));
	

	// Close handles
	CheckCmuError(RSCMU_TDMA_Close (idTDMA1900, cmu_errmsg));
	CheckCmuError(RSCMU_TDMA_Close (idTDMA800, cmu_errmsg));	
	CheckCmuError(RSCMU_Close_Base (idBASE, cmu_errmsg));
	
	return(0);

Error:

  
  return (-1);


}
